@@ -45,14 +45,14 @@
 """
 
 import array
-import unittest
 import warnings
 
 import enchant
-import enchant.tokenize
+from enchant.errors import *
 from enchant.tokenize import get_tokenizer
+from enchant.utils import bytes, unicode, basestring, next
+from enchant.utils import get_default_language
 
-from enchant.utils import bytes, unicode, raw_unicode, basestring
 
 class SpellChecker:
     """Class implementing stateful spellchecking behaviour.
@@ -104,7 +104,7 @@ class SpellChecker:
     """
     _DOC_ERRORS = ["sme","fw","speling","chkr","chkr","chkr"]
     
-    def __init__(self,lang,text=None,tokenize=None,filters=None):
+    def __init__(self,lang=None,text=None,tokenize=None,chunkers=None,filters=None):
         """Constructor for the SpellChecker class.
 
         SpellChecker objects can be created in two ways, depending on
@@ -116,27 +116,34 @@ class SpellChecker:
             
             * text:  to set the text to be checked at creation time
             * tokenize:  a custom tokenization function to use
+            * chunkers:  a list of chunkers to apply during tokenization
             * filters:  a list of filters to apply during tokenization
         
         If <tokenize> is not given and the first argument is a Dict,
         its 'tag' attribute must be a language tag so that a tokenization
-        function can be created automatically.  In particular this means
-        that a PWL dictionary cannot be used without specifying a
-        tokenization function.
+        function can be created automatically.  If this attribute is missing
+        the user's default language will be used.
         """
+        if lang is None:
+            lang = get_default_language()
         if isinstance(lang,basestring):
             dict = enchant.Dict(lang)
         else:
             dict = lang
-            lang = dict.tag
+            try:
+                lang = dict.tag
+            except AttributeError:
+                lang = get_default_language()
+        if lang is None:
+            raise DefaultLanguageNotFoundError
         self.lang = lang
         self.dict = dict
         if tokenize is None:
             try:
-                tokenize = get_tokenizer(lang,filters)
-            except enchant.tokenize.Error:
+                tokenize = get_tokenizer(lang,chunkers,filters)
+            except TokenizerNotFoundError:
                 # Fall back to English tokenization if no match for 'lang'
-                tokenize = get_tokenizer("en",filters)
+                tokenize = get_tokenizer("en",chunkers,filters)
         self._tokenize = tokenize
         
         self.word = None
@@ -237,10 +244,10 @@ class SpellChecker:
         
         """
         # Find the next spelling error.
-        # The uncaught StopIteration from self._tokens.next()
+        # The uncaught StopIteration from next(self._tokens)
         # will provide the StopIteration for this method
         while True:
-            (word,pos) = self._tokens.next()
+            (word,pos) = next(self._tokens)
             # decode back to a regular string
             word = self._array_to_string(word)
             if self.dict.check(word):
@@ -366,144 +373,4 @@ class SpellChecker:
         context = self._text[start:end]
         return self._array_to_string(context)    
         
-            
-class TestChecker(unittest.TestCase):
-    """TestCases for checking behaviour of SpellChecker class."""
-    
-    def test_basic(self):
-        """Test a basic run of the SpellChecker class."""
-        text = """This is sme text with a few speling erors in it. Its gret
-        for checking wheather things are working proprly with the SpellChecker
-        class. Not gret for much elss though."""
-        chkr = SpellChecker("en_US",text=text)
-        for n,err in enumerate(chkr):
-            if n == 0:
-                # Fix up "sme" -> "some" properly
-                self.assertEqual(err.word,"sme")
-                self.assertEqual(err.wordpos,8)
-                self.assert_("some" in err.suggest())
-                err.replace("some")
-            if n == 1:
-                # Ignore "speling"
-                self.assertEqual(err.word,"speling")
-            if n == 2:
-                # Check context around "erors", and replace
-                self.assertEqual(err.word,"erors")
-                self.assertEqual(err.leading_context(5),"ling ")
-                self.assertEqual(err.trailing_context(5)," in i")
-                err.replace(raw_unicode("errors"))
-            if n == 3:
-                # Replace-all on gret as it appears twice
-                self.assertEqual(err.word,"gret")
-                err.replace_always("great")
-            if n == 4:
-                # First encounter with "wheather", move offset back
-                self.assertEqual(err.word,"wheather")
-                err.set_offset(-1*len(err.word))
-            if n == 5:
-                # Second encounter, fix up "wheather'
-                self.assertEqual(err.word,"wheather")
-                err.replace("whether")
-            if n == 6:
-                # Just replace "proprly", but also add an ignore
-                # for "SpellChecker"
-                self.assertEqual(err.word,"proprly")
-                err.replace("properly")
-                err.ignore_always("SpellChecker")
-            if n == 7:
-                # The second "gret" should have been replaced
-                # So it's now on "elss"
-                self.assertEqual(err.word,"elss")
-                err.replace("else")
-            if n > 7:
-                self.fail("Extraneous spelling errors were found")
-        text2 = """This is some text with a few speling errors in it. Its great
-        for checking whether things are working properly with the SpellChecker
-        class. Not great for much else though."""
-        self.assertEqual(chkr.get_text(),text2)
 
-    def test_filters(self):
-        """Test SpellChecker with the 'filters' argument."""
-        text = """I contain WikiWords that ShouldBe skipped by the filters"""
-        chkr = SpellChecker("en_US",text=text,
-                            filters=[enchant.tokenize.WikiWordFilter])
-        for err in chkr:
-            # There are no errors once the WikiWords are skipped
-            self.fail("Extraneous spelling errors were found")
-        self.assertEqual(chkr.get_text(),text)
-        
-    def test_unicode(self):
-        """Test SpellChecker with a unicode string."""
-        text = raw_unicode("""I am a unicode strng with unicode erors.""")
-        chkr = SpellChecker("en_US",text)
-        for n,err in enumerate(chkr):
-            if n == 0:
-                self.assertEqual(err.word,raw_unicode("unicode"))
-                self.assertEqual(err.wordpos,7)
-                chkr.ignore_always()
-            if n == 1:
-                self.assertEqual(err.word,raw_unicode("strng"))
-                chkr.replace_always("string")
-                self.assertEqual(chkr._replace_words[raw_unicode("strng")],raw_unicode("string"))
-            if n == 2:
-                self.assertEqual(err.word,raw_unicode("erors"))
-                chkr.replace("erros")
-                chkr.set_offset(-6)
-            if n == 3:
-                self.assertEqual(err.word,raw_unicode("erros"))
-                chkr.replace("errors")
-        self.assertEqual(n,3)
-        self.assertEqual(chkr.get_text(),raw_unicode("I am a unicode string with unicode errors."))
-
-    def test_chararray(self):
-        """Test SpellChecker with a character array as input."""
-        # Python 3 does not provide 'c' array type
-        if str is unicode:
-           atype = 'u'
-        else:
-           atype = 'c'
-        text = "I wll be stord in an aray"
-        txtarr = array.array(atype,text)
-        chkr = SpellChecker("en_US",txtarr)
-        for (n,err) in enumerate(chkr):
-            if n == 0:
-                self.assertEqual(err.word,"wll")
-                self.assertEqual(err.word.__class__,str)
-            if n == 1:
-                self.assertEqual(err.word,"stord")
-                txtarr[err.wordpos:err.wordpos+len(err.word)] = array.array(atype,"stored")
-                chkr.set_offset(-1*len(err.word))
-            if n == 3:
-                self.assertEqual(err.word,"aray")
-                chkr.replace("array")
-        self.assertEqual(n,3)
-        if str is unicode:
-          self.assertEqual(txtarr.tounicode(),"I wll be stored in an array")
-        else:
-          self.assertEqual(txtarr.tostring(),"I wll be stored in an array")
-
-    def test_pwl(self):
-        """Test checker loop with PWL."""
-        from enchant import DictWithPWL
-        d = DictWithPWL("en_US",None,None)
-        txt = "I am sme text to be cheked with personal list of cheked words"
-        chkr = SpellChecker(d,txt)
-        for n,err in enumerate(chkr):
-            if n == 0:
-                self.assertEqual(err.word,"sme")
-            if n == 1:
-                self.assertEqual(err.word,"cheked")
-                chkr.add()
-        self.assertEqual(n,1)
-
-    def test_bug2785373(self):
-        """Testcases for bug #2785373."""
-        c = SpellChecker(enchant.Dict("en"),"")
-        c.set_text("So, one dey when I wes 17, I left.")
-        for err in c:
-            pass
-        c = SpellChecker(enchant.Dict("en"),"")
-        c.set_text(raw_unicode("So, one dey when I wes 17, I left."))
-        for err in c:
-            pass
- 
