@@ -1,6 +1,6 @@
 # pyenchant
 #
-# Copyright (C) 2004-2008, Ryan Kelly
+# Copyright (C) 2004-2009, Ryan Kelly
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -37,17 +37,16 @@
     strings into words according to the rules of a particular language.
     
     Each tokenization function accepts a string as its only positional
-    argument, and returns an iterator which will yield tuples of the
-    following form, one for each word found:
+    argument, and returns an iterator that yields tuples of the following
+    form, one for each word found:
         
         (<word>,<pos>)
         
     The meanings of these fields should be clear: <word> is the word
     that was found and <pos> is the position within the text at which
     the word began (zero indexed, of course).  The function will work
-    on any string-like object that supports array-slicing - in
-    particular, character-array objects from the 'array' module may
-    be used.
+    on any string-like object that supports array-slicing; in particular
+    character-array objects from the 'array' module may be used.
     
     The iterator also provides the attribute 'offset' which may be used
     to get/set the current position of the tokenizer inside the string
@@ -55,7 +54,11 @@
     have changed during the tokenization process.
     
     To obtain an appropriate tokenization function for the language
-    identified by <tag>, use the function 'get_tokenizer(tag)'.
+    identified by <tag>, use the function 'get_tokenizer(tag)':
+        
+        tknzr = get_tokenizer("en_US")
+        for (word,pos) in tknzr("text to be tokenized goes here")
+            do_something(word)
     
     This library is designed to be easily extendible by third-party
     authors.  To register a tokenization function for the language
@@ -69,34 +72,42 @@
     language.  Based on the author's limited experience, this should
     be at least partially suitable for other languages.
 
-    This module also provides the Filter class and a variety of subclasses.
-    These are designed to allow skipping over certain types of word
-    during the spellchecking process.  You can pass a list of filters to
-    get_tokenizer as follows:
-        
-        tknzr = get_tokenizer("en_US",(URLFilter,WikiWordFilter))
-        
-    Use the tokenizer so returned as follows:
-        
-        tkns = tknzr("text to be tokenized goes here")
-        for (word,pos) in tkns:
-            do_something(word)
+    This module also provides various implementations of "Chunkers" and
+    "Filters".  These classes are designed to make it easy to work with
+    text in a vareity of common formats, by detecting and excluding parts
+    of the text that don't need to be checked.
+
+    A Chunker is a class designed to break a body of text into large chunks
+    of checkable content; for example the HTMLChunker class extracts the 
+    text content from all HTML tags but excludes the tags themselves.
+    A Filter is a class designed to skip individual words during the checking
+    process; for example the URLFilter class skips over any words that
+    have the format of a URL.
+
+    For exmaple, to spellcheck an HTML document it is necessary to split the
+    text into chunks based on HTML tags, and to filter out common word forms
+    such as URLs and WikiWords.  This would look something like the following:
+
+        tknzr = get_tokenier("en_US",(HTMLChunker,),(URLFilter,WikiWordFilter)))
+
+        text = "<html><body>the url is http://example.com</body></html>"
+        for (word,pos) in tknzer(text):
+            ...check each word and react accordingly...
         
 """
 _DOC_ERRORS = ["pos","pos","tknzr","URLFilter","WikiWordFilter",
                "tkns","tknzr","pos","tkns"]
 
-import unittest
 import re
+import warnings
 
 import enchant
+from enchant.utils import next, xrange
+from enchant.errors import *
 
-class Error(enchant.Error):
-    """Exception subclass for the tokenize module.
-    This exception is raised for errors within the enchant.tokenize
-    module.
-    """
-    pass
+#  For backwards-compatability.  This will eventually be removed, but how
+#  does one mark a module-level constant as deprecated?
+Error = TokenizerNotFoundError
 
 
 class tokenize:
@@ -125,6 +136,67 @@ class tokenize:
         return self
 
 
+def get_tokenizer(tag,chunkers=None,filters=None):
+    """Locate an appropriate tokenizer by language tag.
+
+    This requires importing the function 'tokenize' from an
+    appropriate module.  Modules tried are named after the
+    language tag, tried in the following order:
+        * the entire tag (e.g. "en_AU.py")
+        * the base country code of the tag (e.g. "en.py")
+
+    If a suitable function cannot be found, raises TokenizerNotFoundError.
+    
+    If given and not None, 'chunkers' and 'filters' must be lists of chunker
+    classes and filter classes resectively.  These will be applied to the
+    tokenizer during creation.
+    """
+    # "filters" used to be the second argument.  Try to catch cases
+    # where it is given positionally and issue a DeprecationWarning.
+    if chunkers is not None and filters is None:
+        chunkers = list(chunkers)
+        if chunkers:
+            try:
+                chunkers_are_filters = issubclass(chunkers[0],Filter)
+            except TypeError:
+                pass
+            else:
+                if chunkers_are_filters:
+                    msg = "passing 'filters' as a non-keyword argument "\
+                          "to get_tokenizer() is deprecated"
+                    warnings.warn(msg,category=DeprecationWarning)
+                    filters = chunkers
+                    chunkers = None
+    # Ensure only '_' used as separator
+    tag = tag.replace("-","_")
+    # First try the whole tag
+    tkFunc = _try_tokenizer(tag)
+    if tkFunc is None:
+        # Try just the base
+        base = tag.split("_")[0]
+        tkFunc = _try_tokenizer(base)
+        if tkFunc is None:
+            msg = "No tokenizer found for language '%s'" % (tag,)
+            raise TokenizerNotFoundError(msg)
+    # Given the language-specific tokenizer, we now build up the
+    # end result as follows:
+    #    * chunk the text using any given chunkers in turn
+    #    * begin with basic whitespace tokenization
+    #    * apply each of the given filters in turn
+    #    * apply language-specific rules
+    tokenizer = basic_tokenize
+    if chunkers is not None:
+        chunkers = list(chunkers)
+        for i in xrange(len(chunkers)-1,-1,-1):
+            tokenizer = wrap_tokenizer(chunkers[i],tokenizer)
+    if filters is not None:
+        for f in filters:
+            tokenizer = f(tokenizer)
+    tokenizer = wrap_tokenizer(tokenizer,tkFunc)
+    return tokenizer
+get_tokenizer._DOC_ERRORS = ["py","py"]
+
+
 class empty_tokenize(tokenize):
     """Tokenizer class that yields no elements."""
     _DOC_ERRORS = []
@@ -189,47 +261,11 @@ class basic_tokenize(tokenize):
                 return (text[sPos:ePos],sPos)
         raise StopIteration()
 
-
-def get_tokenizer(tag,filters=None):
-    """Locate an appropriate tokenizer by language tag.
-
-    This requires importing the function 'tokenize' from an
-    appropriate module.  Modules tried are named after the
-    language tag, tried in the following order:
-        * the entire tag (e.g. "en_AU.py")
-        * the base country code of the tag (e.g. "en.py")
-
-    If a suitable function cannot be found, raises Error.
-    
-    If given and not None, 'filters' must be a list of filter
-    classes that will be applied to the tokenizer during creation.
-    """
-    # Ensure only '_' used as separator
-    tag = tag.replace("-","_")
-    # First try the whole tag
-    tkFunc = _try_tokenizer(tag)
-    if tkFunc is None:
-        # Try just the base
-        base = tag.split("_")[0]
-        tkFunc = _try_tokenizer(base)
-        if tkFunc is None:
-            raise Error("No tokenizer found for language '%s'" % (tag,))
-    # Given the language-specific tokenizer, we now build up the
-    # end result as follows:
-    #    * begin with basic whitespace tokenization
-    #    * apply each of the given filters in turn
-    #    * apply language-specific rules
-    tokenizer = basic_tokenize
-    if filters is not None:
-        for f in filters:
-            tokenizer = f(tokenizer)
-    tokenizer = wrap_tokenizer(tokenizer,tkFunc)
-    return tokenizer
-get_tokenizer._DOC_ERRORS = ["py","py"]
     
 
 def _try_tokenizer(modName):
     """Look for a tokenizer in the named module.
+
     Returns the function if found, None otherwise.
     """
     modBase = "enchant.tokenize."
@@ -258,6 +294,15 @@ def wrap_tokenizer(tk1,tk2):
 wrap_tokenizer._DOC_ERRORS = ["tk","tk","tk","tk"]
 
 
+class Chunker(tokenize):
+    """Base class for text chunking functions.
+    
+    A chunker is designed to chunk text into large blocks of tokens.  It
+    has the same interface as a tokenizer but is for a different purpose.
+    """
+    pass
+ 
+
 class Filter:
     """Base class for token filtering functions.
     
@@ -267,7 +312,7 @@ class Filter:
       * skip over tokens
       * split tokens into sub-tokens
 
-    Subclasses has two basic options for customising their behaviour.  The
+    Subclasses have two basic options for customising their behaviour.  The
     method _skip(word) may be overridden to return True for words that
     should be skipped, and false otherwise.  The method _split(word) may
     be overridden as tokenization function that will be applied to further
@@ -332,12 +377,12 @@ class Filter:
             # Try to get the next sub-token from word currently being split.
             # If unavailable, move on to the next word and try again.
             try:
-                (word,pos) = self._curtok.next()
+                (word,pos) = next(self._curtok)
                 return (word,pos + self._curpos)
             except StopIteration:
-                (word,pos) = self._tokenizer.next()
+                (word,pos) = next(self._tokenizer)
                 while self._skip(word):
-                    (word,pos) = self._tokenizer.next()
+                    (word,pos) = next(self._tokenizer)
                 self._curword = word
                 self._curpos = pos
                 self._curtok = self._split(word)
@@ -361,7 +406,7 @@ class Filter:
         offset = property(_getOffset,_setOffset)
 
 
-#  Pre-defined filters start here
+#  Pre-defined chunkers and filters start here
 
 class URLFilter(Filter):
     """Filter skipping over URLs.
@@ -407,149 +452,37 @@ class EmailFilter(Filter):
         return False
 
 
-#TODO: HTMLFilter, LaTeXFilter, ...
+class HTMLChunker(Chunker):
+    """Chunker for breaking up HTML documents into chunks of checkable text.
+
+    The operation of this chunker is very simple - anything between a "<"
+    and a ">" will be ignored.  Later versions may improve the algorithm
+    slightly.
+    """
+
+    def next(self):
+        text = self._text
+        offset = self.offset
+        while True:
+            if offset >= len(text):
+                break
+            #  Skip to the end of the current tag, if any.
+            if text[offset] == "<":
+                while text[offset] != ">":
+                    offset += 1
+                offset += 1
+            sPos = offset
+            #  Find the start of the next tag.
+            while offset < len(text) and text[offset] != "<":
+                offset += 1
+            ePos = offset
+            self.offset = offset
+            # Return if chunk isnt empty
+            if(sPos < offset):
+                return (text[sPos:offset],sPos)
+        raise StopIteration()
 
 
-# Test cases begin here
-
-class TestTokenization(unittest.TestCase):
-    """TestCases for testing the basic tokenization functionality."""
-    
-    def test_basic_tokenize(self):
-        """Simple regression test for basic white-space tokenization."""
-        input = """This is a paragraph.  It's not very special, but it's designed
-2 show how the splitter works with many-different combos
-of words. Also need to "test" the (handling) of 'quoted' words."""
-        output = [
-                  ("This",0),("is",5),("a",8),("paragraph",10),("It's",22),
-                  ("not",27),("very",31),("special",36),("but",45),("it's",49),
-                  ("designed",54),("2",63), ("show",65),("how",70),("the",74),
-                  ("splitter",78),("works",87),("with",93),("many-different",98),
-                  ("combos",113),("of",120),("words",123),
-                  ("Also",130),("need",135),
-                  ("to",140),("test",144),("the",150),("handling",155),
-                  ("of",165),("quoted",169),("words",177)
-                 ]
-        self.assertEqual(output,[i for i in basic_tokenize(input)])
-        for (itmO,itmV) in zip(output,basic_tokenize(input)):
-            self.assertEqual(itmO,itmV)
-
-    def test_tokenize_strip(self):
-        """Test special-char-stripping edge-cases in basic_tokenize."""
-        input = "((' <this> \"\" 'text' has (lots) of (special chars} >>]"
-        output = [ ("<this>",4),("text",15),("has",21),("lots",26),("of",32),
-                   ("special",36),("chars}",44),(">>",51)]
-        self.assertEqual(output,[i for i in basic_tokenize(input)])
-        for (itmO,itmV) in zip(output,basic_tokenize(input)):
-            self.assertEqual(itmO,itmV)
-            
-    def test_wrap_tokenizer(self):
-        """Test wrapping of one tokenizer with another."""
-        input = "this-string will be split@according to diff'rnt rules"
-        from enchant.tokenize import en
-        tknzr = wrap_tokenizer(basic_tokenize,en.tokenize)
-        tknzr = tknzr(input)
-        self.assertEquals(tknzr._tokenizer.__class__,basic_tokenize)
-        self.assertEquals(tknzr._tokenizer.offset,0)
-        for (n,(word,pos)) in enumerate(tknzr):
-            if n == 0:
-                self.assertEquals(pos,0)
-                self.assertEquals(word,"this")
-            if n == 1:
-                self.assertEquals(pos,5)
-                self.assertEquals(word,"string")
-            if n == 2:
-                self.assertEquals(pos,12)
-                self.assertEquals(word,"will")
-                # Test setting offset to a previous token
-                tknzr.offset = 5
-                self.assertEquals(tknzr.offset,5)
-                self.assertEquals(tknzr._tokenizer.offset,5)
-                self.assertEquals(tknzr._curtok.__class__,empty_tokenize)
-            if n == 3:
-                self.assertEquals(word,"string")
-                self.assertEquals(pos,5)
-            if n == 4:
-                self.assertEquals(pos,12)
-                self.assertEquals(word,"will")
-            if n == 5:
-                self.assertEquals(pos,17)
-                self.assertEquals(word,"be")
-                # Test setting offset past the current token
-                tknzr.offset = 20
-                self.assertEquals(tknzr.offset,20)
-                self.assertEquals(tknzr._tokenizer.offset,20)
-                self.assertEquals(tknzr._curtok.__class__,empty_tokenize)
-            if n == 6:
-                self.assertEquals(pos,20)
-                self.assertEquals(word,"split")
-            if n == 7:
-                self.assertEquals(pos,26)
-                self.assertEquals(word,"according")
-                # Test setting offset to middle of current token
-                tknzr.offset = 23
-                self.assertEquals(tknzr.offset,23)
-                self.assertEquals(tknzr._tokenizer.offset,23)
-                self.assertEquals(tknzr._curtok.offset,3)
-            if n == 8:
-                self.assertEquals(pos,23)
-                self.assertEquals(word,"it")
-            # OK, I'm pretty happy with the behaviour, no need to
-            # continue testing the rest of the string
+#TODO: LaTeXChunker
 
 
-
-class TestFilters(unittest.TestCase):
-    """TestCases for the various Filter subclasses."""
-    
-    text = """this text with http://url.com and SomeLinksLike
-              ftp://my.site.com.au/some/file AndOthers not:/quite.a.url
-              with-an@aemail.address as well"""
-    
-    def setUp(self):
-        pass
-    
-    def test_URLFilter(self):
-        """Test filtering of URLs"""
-        tkns = get_tokenizer("en_US",(URLFilter,))(self.text)
-        out = [t for t in tkns]
-        exp = [("this",0),("text",5),("with",10),("and",30),
-               ("SomeLinksLike",34),("AndOthers",93),("not",103),("quite",108),
-               ("a",114),("url",116),("with",134),("an",139),("aemail",142),
-               ("address",149),("as",157),("well",160)]
-        self.assertEquals(out,exp)
-        
-    def test_WikiWordFilter(self):
-        """Test filtering of WikiWords"""
-        tkns = get_tokenizer("en_US",(WikiWordFilter,))(self.text)
-        out = [t for t in tkns]
-        exp = [("this",0),("text",5),("with",10),("http",15),("url",22),("com",26),
-               ("and",30), ("ftp",62),("my",68),("site",71),("com",76),("au",80),
-               ("some",83),("file",88),("not",103),("quite",108),
-               ("a",114),("url",116),("with",134),("an",139),("aemail",142),
-               ("address",149),("as",157),("well",160)]
-        self.assertEquals(out,exp)
-        
-    def test_EmailFilter(self):
-        """Test filtering of email addresses"""
-        tkns = get_tokenizer("en_US",(EmailFilter,))(self.text)
-        out = [t for t in tkns]
-        exp = [("this",0),("text",5),("with",10),("http",15),("url",22),("com",26),
-               ("and",30),("SomeLinksLike",34),
-               ("ftp",62),("my",68),("site",71),("com",76),("au",80),
-               ("some",83),("file",88),("AndOthers",93),("not",103),("quite",108),
-               ("a",114),("url",116),
-               ("as",157),("well",160)]
-        self.assertEquals(out,exp)
-        
-    def test_CombinedFilter(self):
-        """Test several filters combined"""
-        tkns=get_tokenizer("en_US",(URLFilter,WikiWordFilter,EmailFilter))(self.text)
-        out = [t for t in tkns]
-        exp = [("this",0),("text",5),("with",10),
-               ("and",30),("not",103),("quite",108),
-               ("a",114),("url",116),
-               ("as",157),("well",160)]
-        self.assertEquals(out,exp)
-        
-        
